## -----------------------------------------------------------------------------
rm(list=ls())                                                                   # Clear the environment 

## -----------------------------------------------------------------------------
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))                     # Set the appropriate environment to work                                                                                # in every machine

## -----------------------------------------------------------------------------
listofpackages <- c("psych","pracma","zoo","lubridate","dplyr",
                    "matrixcalc","matlib","xlsx","readxl","readr","xts","tidyverse","vars","systemfit")
source("../2024_ex1/preparePackages.R")
preparePackages(listofpackages)
rm(listofpackages, preparePackages)

## -----------------------------------------------------------------------------
## Load NS data 
load('data/yields1.rda')
#identify the maturity, in this case 60 quarters one period is one quarter. 
N=60
ttm = c(1:N)*(1/4)
maturities<-c(1:N)

nss_yields<-nss_yields["1980-03-01/2023-06-01"]
dates <- index(nss_yields)
dates_lag <-seq(as.Date("1980-06-01"),length=nrow(nss_yields)-1, by="quarters")

#########################################
# Figure 1-2 yields and returns 
#########################################
ytm_df <- data.frame(dates = index(nss_yields), Data = coredata(nss_yields))
col_names <- c("date", colnames(nss_yields))
colnames(ytm_df) <- col_names
Yields <- as.matrix(ytm_df[,2:61])
T = nrow(Yields)
logPrices = t(t(-Yields)*ttm)
rf =  -1*logPrices[1:T,1]
rx =  logPrices[2:T, 1:(N-1)] - logPrices[1:(T-1), 2:N] - rf[1:T-1]
rx_df<- data.frame(dates = dates_lag, Data = coredata(rx))
colnames_er<- paste("er",seq(2,N),sep = "_")
colnames(rx_df)<- c("date", colnames_er)


long_data <- ytm_df %>%
  gather(key = "Variable", value = "Value", starts_with("n_"))
# Set the levels for the "Variable" factor to ensure the desired color mapping
long_data$Variable <- factor(long_data$Variable, levels = unique(long_data$Variable))
# Define a custom color palette from pale blue to dark blue for the y variables
color_palette <- colorRampPalette(c("lightblue", "darkblue"))(length(unique(long_data$Variable)))

plot <- ggplot(long_data, aes(x = date, y = Value, color = Variable)) +
  geom_line(size = 1) +
  labs(title = "Common Trends in Yields",
       x = "Time", y = "Value") +
  scale_color_manual(values = setNames(color_palette, levels(long_data$Variable))) +  # Manually set colors
  #geom_line(data = subset_df, aes(x = Dates, y = y3m_cc), color = "red", size = 1) +  # Plot x in red
  theme_minimal() +
  theme(axis.line = element_line(color = "black")) +
  scale_x_date(date_breaks = "5 year", date_labels = "%Y") +
  theme(legend.text = element_text(size = 6),  # Adjust the font size in the legend
        legend.key.size = unit(0.1, "cm"))  # Specify a smaller legend key size (1 cm)

print(plot)
ggsave(filename = "figures/Figure_1.pdf", plot = plot, device = "pdf",width = 5.72,height=3.12)

rm(long_data)
long_data <- rx_df %>%
  gather(key = "Variable", value = "Value", starts_with("er_"))

# Set the levels for the "Variable" factor to ensure the desired color mapping
long_data$Variable <- factor(long_data$Variable, levels = unique(long_data$Variable))
# Define a custom color palette from pale blue to dark blue for the y variables
color_palette <- colorRampPalette(c("lightblue", "darkblue"))(length(unique(long_data$Variable)))
plot <- ggplot(long_data, aes(x = date, y = Value, color = Variable)) +
  geom_line(size = 1) +
  labs(title = "Common Cycles in Excess Period Holding Returns",
       x = "Time", y = "Value") +
  scale_color_manual(values = setNames(color_palette, levels(long_data$Variable))) +  # Manually set colors
  #geom_line(data = subset_df, aes(x = Dates, y = y3m_cc), color = "red", size = 1) +  # Plot x in red
  theme_minimal() +
  theme(axis.line = element_line(color = "black")) +
  scale_x_date(date_breaks = "5 year", date_labels = "%Y") +
  theme(legend.text = element_text(size = 6),  # Adjust the font size in the legend
        legend.key.size = unit(0.1, "cm"))  # Specify a smaller legend key size (1 cm)

print(plot)
ggsave(filename = "figures/Figure_2.pdf", plot = plot, device = "pdf",width = 5.72,height=3.12)
rm(plot,rx,rx_df,ytm_df,long_data,logPrices,Yields)


#########################################
# Factors
#########################################

load('data/output/X_ACM.rda')
load('data/output/ACM.rda')
X.df_long <- gather(X.df, key = "Variable", value = "Value", -Date)

# Create the time-series plot with a legend
plot <-ggplot(data = X.df_long, aes(x = Date, y = Value, color = Variable)) +
  geom_line() +
  labs(title = "The first five PC extracted from yields",
       x = "Date",
       y = "Value") +
  scale_color_manual(values = c("PC1" = "red", "PC2" = "blue", "PC3" = "green", "PC4" = "purple", "PC5" = "orange")) +
  theme_minimal() +
  theme(legend.title = element_blank())
print(plot)
ggsave("figures/fig_X.pdf", plot = plot,width = 5.72, height = 3.12)

#######################
# TERM PREMIA
#######################

plot<- ggplot(ACM, aes(x = Date)) +
  geom_line(aes(y = TP40, color = "TP40"), size = 1) +
  geom_line(aes(y = TP4, color = "TP4"), linetype = "dashed", size = 1) +
  labs(x = "Date", y = "Term PRemia", title = "1Y and 10Y Term Premia in the standard model") +
  scale_color_manual(values = c("TP40" = "blue", "TP4" = "red")) +
  theme_minimal() +
  theme(
    legend.position = "top",
    plot.title = element_text(hjust = 0.5)  # Center the title
  ) +
  guides(color = guide_legend(title = "Series"))
print(plot)
ggsave("figures/fig_ACM_tp.pdf", plot = plot)

#######################
# FIT and FORECAST
#######################

plotforc10y <- data.frame(data.frame(ACM$Date,ACM$n_40,ACM$n_40for,ACM$n_40fit
) )

library(data.table)

# Convert the data frame to a data.table
setDT(plotforc10y)
# Create the n_40plot series
plotforc10y[, ACM.n_40plot := ifelse(ACM.Date <= as.Date("2005-12-01"), ACM.n_40fit, ACM.n_40for)]

plot<-ggplot(plotforc10y, aes(x = ACM.Date)) +
  geom_line(aes(y = ACM.n_40, color = "n_40"), size = 1) +
  geom_line(aes(y = ACM.n_40plot, color = "Standard Model"), linetype = "dashed", size = 1) +
  labs(x = "Date", y = "Fit and Forecast", title = "10-year Yields") +
  scale_color_manual(values = c("n_40" = "blue", "Standard Model" = "red")) +
  theme_minimal() +
  theme(
    legend.position = "top",
    plot.title = element_text(hjust = 0.5)  # Center the title
  ) +
  guides(color = guide_legend(title = ""))
plot <- plot +
  geom_vline(xintercept = as.Date("2005-12-01"), color = "black")


# Shading the area from 2006 onwards
# plot <- plot +
#   geom_rect(
#     xmin = as.Date("2006-01-01"),
#     xmax = max(plotforc10y$ACM.Date),
#     ymin = -Inf,
#     ymax = Inf,
#     fill = "gray",
#     alpha = 0.01,
#     color = NA  # Set the outline color to match the fill color
#   )
print(plot)
ggsave("figures/fig_fitforc_10y.pdf", plot = plot,width = 5.72, height = 3.50)


plotforc3m <- data.frame(data.frame(ACM$Date,ACM$n_1,ACM$n_1for,
                                    ACM$n_1fit
) )                                      

# Convert the data frame to a data.table
setDT(plotforc3m)
# Create the n_40plot series
plotforc3m[, ACM.n_1plot := ifelse(ACM.Date <= as.Date("2005-12-01"), ACM.n_1fit, ACM.n_1for)]

plot<-ggplot(plotforc3m, aes(x = ACM.Date)) +
  geom_line(aes(y = ACM.n_1, color = "n_1"), size = 1) +
  geom_line(aes(y = ACM.n_1plot, color = "Standard Model"), linetype = "dashed", size = 1) +
  labs(x = "Date", y = "Fit and Forecast", title = "3-month Yields") +
  scale_color_manual(values = c("n_1" = "blue", "Standard Model" = "red")) +
  theme_minimal() +
  theme(
    legend.position = "top",
    plot.title = element_text(hjust = 0.5)  # Center the title
  ) +
  guides(color = guide_legend(title = ""))
plot <- plot +
  geom_vline(xintercept = as.Date("2005-12-01"), color = "black")


# Shading the area from 2006 onwards
# plot <- plot +
#   geom_rect(
#     xmin = as.Date("2006-01-01"),
#     xmax = max(plotforc3m$ACM.Date),
#     ymin = -Inf,
#     ymax = Inf,
#     fill = "gray",
#     alpha = 0.01,
#     color = NA  # Set the outline color to match the fill color
#   )
print(plot)
ggsave("figures/fig_fitforc_3m.pdf", plot = plot,width = 5.72, height = 3.50)
